@@ -1,4 +1,4 @@
-// Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
+// Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 
 use std::sync::Arc;
 
@@ -13,11 +13,11 @@ use deno_ast::SourceTextInfo;
 use deno_graph::SourceParser;
 
 #[derive(Default)]
-pub struct ScopeAnalysisParser {}
+pub struct ScopeAnalysisParser;
 
 impl ScopeAnalysisParser {
   pub fn new() -> Self {
-    ScopeAnalysisParser {}
+    ScopeAnalysisParser
   }
 }
 
