@@ -1,4 +1,4 @@
-// Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
+// Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 
 use std::path::Path;
 use std::path::PathBuf;
@@ -6,19 +6,13 @@ use std::path::PathBuf;
 use anyhow::Result;
 use deno_ast::ModuleSpecifier;
 
+pub const BOM_CHAR: char = '\u{FEFF}';
+
 pub fn get_relative_specifier(
   from: impl AsRef<Path>,
   to: impl AsRef<Path>,
 ) -> String {
-  let mut relative_path = get_relative_path(from, to);
-  // no need to check media type because the extension should have been resolved
-  // based on the media type at this point
-  if is_json_file_path(&relative_path) {
-    relative_path.set_extension("json");
-  } else {
-    relative_path.set_extension("js");
-  }
-
+  let relative_path = get_relative_path(from, to).with_extension("js");
   let relative_path_str = relative_path
     .to_string_lossy()
     .to_string()
@@ -60,15 +54,6 @@ pub fn url_to_file_path(module_specifier: &ModuleSpecifier) -> Result<PathBuf> {
   Ok(PathBuf::from(final_text))
 }
 
-pub fn is_json_file_path(path: impl AsRef<Path>) -> bool {
-  path
-    .as_ref()
-    .extension()
-    .map(|e| e.to_string_lossy().to_lowercase())
-    .unwrap_or_default()
-    == "json"
-}
-
 fn is_windows_path_segment(specifier: &str) -> bool {
   let mut chars = specifier.chars();
 
@@ -83,3 +68,12 @@ fn is_windows_path_segment(specifier: &str) -> bool {
 
   chars.next().is_none()
 }
+
+/// Strips the byte order mark from the provided text if it exists.
+pub fn strip_bom(text: &str) -> &str {
+  if text.starts_with(BOM_CHAR) {
+    &text[BOM_CHAR.len_utf8()..]
+  } else {
+    text
+  }
+}
