@@ -1,8 +1,12 @@
-// Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
+// Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 
 use std::path::PathBuf;
 
 use deno_node_transform::Dependency;
+use deno_node_transform::GlobalName;
+use deno_node_transform::MappedSpecifier;
+use deno_node_transform::ScriptTarget;
+use deno_node_transform::Shim;
 use pretty_assertions::assert_eq;
 
 #[macro_use]
@@ -27,66 +31,199 @@ async fn transform_standalone_file() {
 }
 
 #[tokio::test]
-async fn transform_deno_shim() {
+async fn transform_shims() {
   assert_transforms(vec![
     (
       "Deno.readTextFile();",
       concat!(
-        r#"import * as denoShim from "test-shim";"#,
-        "\ndenoShim.Deno.readTextFile();"
+        r#"import * as dntShim from "./_dnt.shims.js";"#,
+        "\ndntShim.Deno.readTextFile();"
       ),
     ),
     (
       "const [test=Deno] = other;",
       concat!(
-        r#"import * as denoShim from "test-shim";"#,
-        "\nconst [test=denoShim.Deno] = other;"
+        r#"import * as dntShim from "./_dnt.shims.js";"#,
+        "\nconst [test=dntShim.Deno] = other;"
       ),
     ),
     (
       "const obj = { test: Deno };",
       concat!(
-        r#"import * as denoShim from "test-shim";"#,
-        "\nconst obj = { test: denoShim.Deno };"
+        r#"import * as dntShim from "./_dnt.shims.js";"#,
+        "\nconst obj = { test: dntShim.Deno };"
       ),
     ),
     (
       concat!(
-        "const decl01 = Blob;\n",
-        "const decl02 = crypto;\n",
-        "const decl03 = fetch;\n",
-        "const decl04 = File;\n",
-        "const decl05 = FormData;\n",
-        "const decl06 = Headers;\n",
-        "const decl07 = Request;\n",
-        "const decl08 = Response;\n",
-        "const decl09 = alert;\n",
-        "const decl10 = confirm;\n",
-        "const decl11: typeof prompt = prompt;\n",
+        "const decl01 = Deno;\n",
+        "const decl02 = setTimeout;\n",
+        "const decl03 = setInterval;\n",
+        "const decl04: typeof setTimeout = setTimeout;\n",
         "setTimeout(() => {}, 100);\n",
-        "setInterval(() => {}, 100);\n",
+        "if ('test' in Deno) {}\n",
       ),
       concat!(
-        r#"import * as denoShim from "test-shim";"#,
-        "\nconst decl01 = denoShim.Blob;\n",
-        "const decl02 = denoShim.crypto;\n",
-        "const decl03 = denoShim.fetch;\n",
-        "const decl04 = denoShim.File;\n",
-        "const decl05 = denoShim.FormData;\n",
-        "const decl06 = denoShim.Headers;\n",
-        "const decl07 = denoShim.Request;\n",
-        "const decl08 = denoShim.Response;\n",
-        "const decl09 = denoShim.alert;\n",
-        "const decl10 = denoShim.confirm;\n",
-        "const decl11: typeof denoShim.prompt = denoShim.prompt;\n",
-        "denoShim.setTimeout(() => {}, 100);\n",
-        "denoShim.setInterval(() => {}, 100);\n",
+        r#"import * as dntShim from "./_dnt.shims.js";"#,
+        "\nconst decl01 = dntShim.Deno;\n",
+        "const decl02 = dntShim.setTimeout;\n",
+        "const decl03 = dntShim.setInterval;\n",
+        "const decl04: typeof dntShim.setTimeout = dntShim.setTimeout;\n",
+        "dntShim.setTimeout(() => {}, 100);\n",
+        "if ('test' in dntShim.Deno) {}\n",
       ),
     ),
   ])
   .await;
 }
 
+#[tokio::test]
+async fn transform_shim_custom_shims() {
+  let result = TestBuilder::new()
+    .with_loader(|loader| {
+      loader.add_local_file(
+        "/mod.ts",
+        "fetch(); console.log(Blob); fetchTest(); DOMException;",
+      );
+    })
+    .add_shim(Shim {
+      package: MappedSpecifier {
+        name: "node-fetch".to_string(),
+        version: Some("~3.1.0".to_string()),
+        sub_path: None,
+      },
+      types_package: None,
+      global_names: vec![GlobalName {
+        name: "fetch".to_string(),
+        export_name: Some("default".to_string()),
+        type_only: false,
+      }],
+    })
+    .add_shim(Shim {
+      package: MappedSpecifier {
+        name: "node-fetch".to_string(),
+        version: Some("~3.1.0".to_string()),
+        sub_path: Some("test".to_string()),
+      },
+      types_package: None,
+      global_names: vec![GlobalName {
+        name: "fetchTest".to_string(),
+        export_name: Some("fetchTestName".to_string()),
+        type_only: false,
+      }],
+    })
+    .add_shim(Shim {
+      package: MappedSpecifier {
+        name: "domexception".to_string(),
+        version: Some("^4.0.0".to_string()),
+        sub_path: None,
+      },
+      types_package: Some(Dependency {
+        name: "@types/domexception".to_string(),
+        version: "^2.0.1".to_string(),
+      }),
+      global_names: vec![GlobalName {
+        name: "DOMException".to_string(),
+        export_name: Some("default".to_string()),
+        type_only: false,
+      }],
+    })
+    .add_shim(Shim {
+      package: MappedSpecifier {
+        name: "buffer".to_string(),
+        version: None,
+        sub_path: None,
+      },
+      types_package: None,
+      global_names: vec![
+        GlobalName {
+          name: "Blob".to_string(),
+          export_name: None,
+          type_only: false,
+        },
+        GlobalName {
+          name: "Other".to_string(),
+          export_name: None,
+          type_only: true,
+        },
+      ],
+    })
+    .add_shim(Shim {
+      package: MappedSpecifier {
+        name: "type-only".to_string(),
+        version: None,
+        sub_path: None,
+      },
+      types_package: None,
+      global_names: vec![GlobalName {
+        name: "TypeOnly".to_string(),
+        export_name: None,
+        type_only: true,
+      }],
+    })
+    .transform()
+    .await
+    .unwrap();
+
+  assert_files!(
+    result.main.files,
+    &[
+      (
+        "_dnt.shims.ts",
+        get_shim_file_text(
+          concat!(
+            "import { default as fetch } from \"node-fetch\";\n",
+            "export { default as fetch } from \"node-fetch\";\n",
+            "import { fetchTestName as fetchTest } from \"node-fetch/test\";\n",
+            "export { fetchTestName as fetchTest } from \"node-fetch/test\";\n",
+            "import { default as DOMException } from \"domexception\";\n",
+            "export { default as DOMException } from \"domexception\";\n",
+            "import { Blob } from \"buffer\";\n",
+            "export { Blob, type Other } from \"buffer\";\n",
+            "export { type TypeOnly } from \"type-only\";\n",
+            "\n",
+            "const dntGlobals = {\n",
+            "  fetch,\n",
+            "  fetchTest,\n",
+            "  DOMException,\n",
+            "  Blob,\n",
+            "};\n",
+            "export const dntGlobalThis = createMergeProxy(globalThis, dntGlobals);\n",
+          ).to_string(),
+        ),
+      ),
+      (
+        "mod.ts",
+        concat!(
+          "import * as dntShim from \"./_dnt.shims.js\";\n",
+          "dntShim.fetch(); console.log(dntShim.Blob); dntShim.fetchTest(); dntShim.DOMException;"
+        )
+        .to_string()
+      ),
+    ]
+  );
+  assert_eq!(
+    result.main.dependencies,
+    vec![
+      Dependency {
+        name: "node-fetch".to_string(),
+        version: "~3.1.0".to_string(),
+      },
+      Dependency {
+        name: "domexception".to_string(),
+        version: "^4.0.0".to_string(),
+      }
+    ]
+  );
+  assert_eq!(
+    result.test.dependencies,
+    vec![Dependency {
+      name: "@types/domexception".to_string(),
+      version: "^2.0.1".to_string(),
+    }]
+  );
+}
+
 #[tokio::test]
 async fn no_transform_deno_ignored() {
   assert_identity_transforms(vec!["// dnt-shim-ignore\nDeno.readTextFile();"])
@@ -113,32 +250,67 @@ async fn transform_legacy_deno_shim_ignore_warnings() {
 }
 
 #[tokio::test]
-async fn transform_global_this_deno() {
+async fn transform_global_this_shim() {
+  assert_transforms(vec![(
+    concat!(
+      "globalThis.Deno.readTextFile();",
+      "globalThis.test();",
+      "globalThis.test.test();",
+      "globalThis['test']();",
+      r#"globalThis["test"]();"#,
+      "globalThis.Deno = 5;",
+      "true ? globalThis : globalThis;",
+      "typeof globalThis.Deno;",
+      "'Deno' in globalThis;",
+      "typeof globalThis;",
+      "globalThis == null;",
+      "globalThis ? true : false;",
+      "type Test1 = typeof globalThis;",
+      "type Test2 = typeof globalThis.Window;",
+      "type Test3 = typeof globalThis.Deno;",
+      "type Test4 = window.Something;",
+    ),
+    concat!(
+      r#"import * as dntShim from "./_dnt.shims.js";"#,
+      "\ndntShim.dntGlobalThis.Deno.readTextFile();",
+      "globalThis.test();",
+      "globalThis.test.test();",
+      "globalThis['test']();",
+      r#"globalThis["test"]();"#,
+      "dntShim.dntGlobalThis.Deno = 5;",
+      "true ? dntShim.dntGlobalThis : dntShim.dntGlobalThis;",
+      "typeof dntShim.dntGlobalThis.Deno;",
+      "'Deno' in dntShim.dntGlobalThis;",
+      "typeof dntShim.dntGlobalThis;",
+      "dntShim.dntGlobalThis == null;",
+      "dntShim.dntGlobalThis ? true : false;",
+      "type Test1 = typeof globalThis;",
+      "type Test2 = typeof globalThis.Window;",
+      "type Test3 = typeof dntShim.Deno;",
+      "type Test4 = globalThis.Something;",
+    ),
+  )])
+  .await;
+}
+
+#[tokio::test]
+async fn transform_window() {
   assert_transforms(vec![
     (
+      concat!("window.test = 5;", "window.Deno.test();",),
       concat!(
-        "globalThis.Deno.readTextFile();",
-        "globalThis.test();",
-        "globalThis.test.test();",
-        "globalThis['test']();",
-        r#"globalThis["test"]();"#,
-        "globalThis.Deno = 5;",
-        "true ? globalThis : globalThis;",
-        "typeof globalThis.Deno;",
+        r#"import * as dntShim from "./_dnt.shims.js";"#,
+        "\nglobalThis.test = 5;",
+        "dntShim.dntGlobalThis.Deno.test();",
       ),
-      concat!(
-        r#"import * as denoShim from "test-shim";"#,
-        "\n({ ...denoShim, ...globalThis }).Deno.readTextFile();",
-        "globalThis.test();",
-        "globalThis.test.test();",
-        "globalThis['test']();",
-        r#"globalThis["test"]();"#,
-        "globalThis.Deno = 5;",
-        "true ? ({ ...denoShim, ...globalThis }) : ({ ...denoShim, ...globalThis });",
-        "typeof ({ ...denoShim, ...globalThis }).Deno;"
-      )
     ),
-  ]).await;
+    (
+      // should be as-is because there's a declaration
+      "const window = {}; window.test;",
+      "const window = {}; window.test;",
+    ),
+  ])
+  .await;
 }
 
 #[tokio::test]
@@ -171,10 +343,6 @@ async fn no_shim_situations() {
     "export { Deno as test } from 'test';",
     "try {} catch (Deno) {}",
     "function test(Deno) {}",
-    "typeof globalThis;",
-    "globalThis == null;",
-    "globalThis ? true : false;",
-    "type Test = typeof globalThis;",
     "interface Response {} function test(r: Response) {}",
   ])
   .await;
@@ -190,9 +358,9 @@ async fn transform_deno_collision() {
       "Deno.test;"
     ),
     concat!(
-      r#"import * as denoShim from "test-shim";"#,
+      r#"import * as dntShim from "./_dnt.shims.js";"#,
       "\nconst Deno = {};",
-      "const { Deno: Deno2 } = ({ ...denoShim, ...globalThis });",
+      "const { Deno: Deno2 } = dntShim.dntGlobalThis;",
       "Deno2.readTextFile();",
       "Deno.test;"
     ),
@@ -728,7 +896,10 @@ async fn transform_specifier_mappings() {
           "/mod.ts",
           concat!(
             "import * as remote from 'http://localhost/mod.ts';\n",
-            "import * as local from './file.ts';\n"
+            "import * as local from './file.ts';\n",
+            "import * as entryA from 'http://localhost/mod/entryA.ts';\n",
+            "import * as entryB from 'http://localhost/mod/entryB.ts';\n",
+            "import * as entryC from 'http://localhost/mod/entryC.ts';\n",
           ),
         )
         .add_remote_file(
@@ -740,24 +911,56 @@ async fn transform_specifier_mappings() {
       "http://localhost/mod.ts",
       "remote-module",
       Some("1.0.0"),
+      None,
+    )
+    .add_specifier_mapping("file:///file.ts", "local-module", None, None)
+    .add_specifier_mapping(
+      "http://localhost/mod/entryA.ts",
+      "mod",
+      Some("~0.1.0"),
+      None,
+    )
+    .add_specifier_mapping(
+      "http://localhost/mod/entryB.ts",
+      "mod",
+      Some("~0.1.0"),
+      Some("entryB"),
+    )
+    .add_specifier_mapping(
+      "http://localhost/mod/entryC.ts",
+      "mod",
+      Some("~0.1.0"),
+      Some("other/entryC.js"),
     )
-    .add_specifier_mapping("file:///file.ts", "local-module", None)
     .transform()
     .await
     .unwrap();
 
   assert_files!(
     result.main.files,
-    &[
-      ("mod.ts", "import * as remote from 'remote-module';\nimport * as local from 'local-module';\n"),
-    ]
+    &[(
+      "mod.ts",
+      concat!(
+        "import * as remote from 'remote-module';\n",
+        "import * as local from 'local-module';\n",
+        "import * as entryA from 'mod';\n",
+        "import * as entryB from 'mod/entryB';\n",
+        "import * as entryC from 'mod/other/entryC.js';\n",
+      )
+    )]
   );
   assert_eq!(
     result.main.dependencies,
-    &[Dependency {
-      name: "remote-module".to_string(),
-      version: "1.0.0".to_string(),
-    },]
+    &[
+      Dependency {
+        name: "mod".to_string(),
+        version: "~0.1.0".to_string(),
+      },
+      Dependency {
+        name: "remote-module".to_string(),
+        version: "1.0.0".to_string(),
+      }
+    ]
   );
 }
 
@@ -767,8 +970,18 @@ async fn transform_not_found_mappings() {
     .with_loader(|loader| {
       loader.add_local_file("/mod.ts", "test");
     })
-    .add_specifier_mapping("http://localhost/mod.ts", "local-module", None)
-    .add_specifier_mapping("http://localhost/mod2.ts", "local-module2", None)
+    .add_specifier_mapping(
+      "http://localhost/mod.ts",
+      "local-module",
+      None,
+      None,
+    )
+    .add_specifier_mapping(
+      "http://localhost/mod2.ts",
+      "local-module2",
+      None,
+      None,
+    )
     .transform()
     .await
     .err()
@@ -823,6 +1036,8 @@ async fn skypack_esm_module_mapping() {
             // custom esm.sh stuff like this should download the dependency
             "import package4 from 'https://esm.sh/swr?deps=react@16.14.0';\n",
             "import package5 from 'https://esm.sh/test@1.2.5?deps=react@16.14.0';\n",
+            "import package6 from 'https://cdn.skypack.dev/preact@^10.5.0/hooks?dts';\n",
+            "import package7 from 'https://esm.sh/react-dom@17.0.2/server';\n",
           ),
         )
         .add_remote_file_with_headers(
@@ -849,11 +1064,13 @@ async fn skypack_esm_module_mapping() {
           "import package2 from '@scope/package-name';\n",
           "import package3 from 'react';\n",
           "import package4 from './deps/esm_sh/swr.js';\n",
-          "import package5 from './deps/esm_sh/test_1.js';\n"
+          "import package5 from './deps/esm_sh/test_1.2.5.js';\n",
+          "import package6 from 'preact/hooks';\n",
+          "import package7 from 'react-dom/server';\n",
         )
       ),
       ("deps/esm_sh/swr.ts", "",),
-      ("deps/esm_sh/test_1.ts", "",)
+      ("deps/esm_sh/test_1.2.5.ts", "",)
     ]
   );
   assert_eq!(
@@ -871,6 +1088,10 @@ async fn skypack_esm_module_mapping() {
         name: "react".to_string(),
         version: "17.0.2".to_string(),
       },
+      Dependency {
+        name: "react-dom".to_string(),
+        version: "17.0.2".to_string(),
+      }
     ]
   );
 }
@@ -898,6 +1119,44 @@ async fn skypack_module_mapping_different_versions() {
   );
 }
 
+#[tokio::test]
+async fn transform_import_map() {
+  let result = TestBuilder::new()
+    .with_loader(|loader| {
+      loader
+        .add_local_file(
+          "/mod.ts",
+          "import * as remote from 'localhost/mod.ts';",
+        )
+        .add_local_file(
+          "/import_map.json",
+          r#"{
+  "imports": {
+    "localhost/": "/subdir/"
+  }
+}"#,
+        )
+        .add_local_file(
+          "/subdir/mod.ts",
+          "import * as myOther from './other.ts';",
+        )
+        .add_local_file("/subdir/other.ts", "export function test() {}");
+    })
+    .set_import_map("file:///import_map.json")
+    .transform()
+    .await
+    .unwrap();
+
+  assert_files!(
+    result.main.files,
+    &[
+      ("mod.ts", "import * as remote from './subdir/mod.js';",),
+      ("subdir/mod.ts", "import * as myOther from './other.js';",),
+      ("subdir/other.ts", "export function test() {}",)
+    ]
+  );
+}
+
 #[tokio::test]
 async fn transform_multiple_entry_points() {
   let result = TestBuilder::new()
@@ -942,6 +1201,7 @@ async fn test_entry_points() {
         );
     })
     .add_test_entry_point("file:///mod.test.ts")
+    .add_default_shims()
     .transform()
     .await
     .unwrap();
@@ -958,30 +1218,60 @@ async fn test_entry_points() {
     },]
   );
   assert_eq!(result.main.entry_points, &[PathBuf::from("mod.ts")]);
-  assert_eq!(result.main.shim_used, false);
 
   assert_files!(
     result.test.files,
-    &[(
-      "mod.test.ts",
-      concat!(
-        "import * as denoShim from \"deno.ns\";\n",
-        "import './mod.js';\n",
-        "import package1 from 'preact';\n",
-        "import package3 from 'react';\n",
-        "denoShim.Deno.writeTextFile('test', 'test')"
+    &[
+      (
+        "mod.test.ts",
+        concat!(
+          "import * as dntShim from \"./_dnt.test_shims.js\";\n",
+          "import './mod.js';\n",
+          "import package1 from 'preact';\n",
+          "import package3 from 'react';\n",
+          "dntShim.Deno.writeTextFile('test', 'test')"
+        )
+        .to_string(),
       ),
-    )]
+      (
+        "_dnt.test_shims.ts",
+        get_shim_file_text(
+          concat!(
+            "import { Deno } from \"@deno/shim-deno\";\n",
+            "export { Deno } from \"@deno/shim-deno\";\n",
+            "import { setTimeout, setInterval } from \"@deno/shim-timers\";\n",
+            "export { setTimeout, setInterval } from \"@deno/shim-timers\";\n",
+            "\n",
+            "const dntGlobals = {\n",
+            "  Deno,\n",
+            "  setTimeout,\n",
+            "  setInterval,\n",
+            "};\n",
+            "export const dntGlobalThis = createMergeProxy(globalThis, dntGlobals);\n",
+          )
+          .to_string(),
+        ),
+      )
+    ]
   );
   assert_eq!(
     result.test.dependencies,
-    &[Dependency {
-      name: "react".to_string(),
-      version: "17.0.2".to_string(),
-    },]
+    &[
+      Dependency {
+        name: "react".to_string(),
+        version: "17.0.2".to_string(),
+      },
+      Dependency {
+        name: "@deno/shim-deno".to_string(),
+        version: "^0.1.0".to_string(),
+      },
+      Dependency {
+        name: "@deno/shim-timers".to_string(),
+        version: "^0.1.0".to_string(),
+      }
+    ]
   );
   assert_eq!(result.test.entry_points, &[PathBuf::from("mod.test.ts")]);
-  assert_eq!(result.test.shim_used, true);
 }
 
 #[tokio::test]
@@ -1045,18 +1335,16 @@ async fn test_entry_points_same_module_multiple_places() {
     ]
   );
   assert_eq!(result.main.entry_points, &[PathBuf::from("mod.ts")]);
-  assert_eq!(result.main.shim_used, false);
 
   assert_files!(
     result.test.files,
     &[("mod.test.ts", "import * as deps from './deps.js';",)]
   );
   assert_eq!(result.test.entry_points, &[PathBuf::from("mod.test.ts")]);
-  assert_eq!(result.test.shim_used, false);
 }
 
 #[tokio::test]
-async fn polyfills() {
+async fn polyfills_all() {
   let result = TestBuilder::new()
     .with_loader(|loader| {
       loader
@@ -1068,6 +1356,7 @@ async fn polyfills() {
             "} catch (err) {\n",
             "  err.cause = new Error();\n",
             "}\n",
+            "''.replaceAll('test', 'other');\n",
           ),
         )
         .add_local_file("/mod.test.ts", "import * as mod from './mod.ts';");
@@ -1089,26 +1378,76 @@ async fn polyfills() {
           "} catch (err) {\n",
           "  err.cause = new Error();\n",
           "}\n",
+          "''.replaceAll('test', 'other');\n",
         ),
       ),
       (
         "_dnt.polyfills.ts",
         concat!(
-          include_str!("../src/polyfills/scripts/object-has-own.ts"),
-          include_str!("../src/polyfills/scripts/error-cause.ts"),
+          include_str!("../src/polyfills/scripts/esnext.object-has-own.ts"),
+          include_str!("../src/polyfills/scripts/esnext.error-cause.ts"),
+          include_str!("../src/polyfills/scripts/es2021.string-replaceAll.ts"),
         )
       ),
     ]
   );
   assert_eq!(result.main.entry_points, &[PathBuf::from("mod.ts")]);
-  assert_eq!(result.main.shim_used, false);
 
   assert_files!(
     result.test.files,
     &[("mod.test.ts", concat!("import * as mod from './mod.js';",),)]
   );
   assert_eq!(result.test.entry_points, &[PathBuf::from("mod.test.ts")]);
-  assert_eq!(result.test.shim_used, false);
+}
+
+#[tokio::test]
+async fn polyfills_string_replaceall_target() {
+  test_string_replace_all_polyfill(ScriptTarget::ES2020, true).await;
+  test_string_replace_all_polyfill(ScriptTarget::ES2021, false).await;
+}
+
+async fn test_string_replace_all_polyfill(
+  target: ScriptTarget,
+  should_have_polyfill: bool,
+) {
+  let result = TestBuilder::new()
+    .with_loader(|loader| {
+      loader
+        .add_local_file("/mod.ts", "''.replaceAll('test', 'other');\n")
+        .add_local_file("/mod.test.ts", "import * as mod from './mod.ts';");
+    })
+    .add_test_entry_point("file:///mod.test.ts")
+    .set_target(target)
+    .transform()
+    .await
+    .unwrap();
+
+  if should_have_polyfill {
+    assert_files!(
+      result.main.files,
+      &[
+        (
+          "mod.ts",
+          concat!(
+            "import './_dnt.polyfills.js';\n",
+            "''.replaceAll('test', 'other');\n",
+          ),
+        ),
+        (
+          "_dnt.polyfills.ts",
+          concat!(include_str!(
+            "../src/polyfills/scripts/es2021.string-replaceAll.ts"
+          ),)
+        ),
+      ]
+    );
+  } else {
+    assert_files!(
+      result.main.files,
+      &[("mod.ts", "''.replaceAll('test', 'other');\n",)]
+    );
+  }
+  assert_eq!(result.main.entry_points, &[PathBuf::from("mod.ts")]);
 }
 
 #[tokio::test]
@@ -1126,7 +1465,6 @@ async fn polyfills_test_files() {
 
   assert_files!(result.main.files, &[("mod.ts", "",)]);
   assert_eq!(result.main.entry_points, &[PathBuf::from("mod.ts")]);
-  assert_eq!(result.main.shim_used, false);
 
   assert_files!(
     result.test.files,
@@ -1140,12 +1478,11 @@ async fn polyfills_test_files() {
       ),
       (
         "_dnt.test_polyfills.ts",
-        include_str!("../src/polyfills/scripts/object-has-own.ts"),
+        include_str!("../src/polyfills/scripts/esnext.object-has-own.ts"),
       )
     ]
   );
   assert_eq!(result.test.entry_points, &[PathBuf::from("mod.test.ts")]);
-  assert_eq!(result.test.shim_used, false);
 }
 
 #[tokio::test]
@@ -1199,12 +1536,11 @@ async fn redirects_general() {
       ("myFunction.ts", "export function myFunction() {}",),
       (
         "_dnt.polyfills.ts",
-        include_str!("../src/polyfills/scripts/object-has-own.ts")
+        include_str!("../src/polyfills/scripts/esnext.object-has-own.ts")
       ),
     ]
   );
   assert_eq!(result.main.entry_points, &[PathBuf::from("mod.ts")]);
-  assert_eq!(result.main.shim_used, false);
 }
 
 #[tokio::test]
@@ -1223,7 +1559,6 @@ async fn redirect_entrypoint() {
 
   assert_files!(result.main.files, &[("mod.node.ts", "5;")]);
   assert_eq!(result.main.entry_points, &[PathBuf::from("mod.node.ts")]);
-  assert_eq!(result.main.shim_used, false);
 }
 
 #[tokio::test]
@@ -1256,7 +1591,7 @@ async fn json_module_import_default() {
           "/mod.ts",
           r#"import jsonData from './data.json' assert { type: 'json' };"#,
         )
-        .add_local_file("/data.json", r#"{ "prop": 5 }"#);
+        .add_local_file("/data.json", "\u{FEFF}{ \"prop\": 5 }");
     })
     .transform()
     .await
@@ -1264,7 +1599,10 @@ async fn json_module_import_default() {
 
   assert_files!(
     result.main.files,
-    &[("mod.ts", r#"const jsonData = JSON.parse(`{ "prop": 5 }`);"#),]
+    &[
+      ("mod.ts", r#"import jsonData from './data.js';"#),
+      ("data.js", r#"export default JSON.parse(`{ "prop": 5 }`);"#)
+    ]
   );
 }
 
@@ -1275,7 +1613,7 @@ async fn json_module_dynamic_import() {
       loader
         .add_local_file(
           "/mod.ts",
-          r#"const jsonData = await import('./data.json', { assert: { type: 'json' } });"#
+          r#"const jsonData = (await import('./data.json', { assert: { type: 'json' } })).default;"#
         )
         .add_local_file("/data.json", r#"{ "prop": 5 }"#);
     })
@@ -1285,16 +1623,19 @@ async fn json_module_dynamic_import() {
 
   assert_files!(
     result.main.files,
-    &[(
-      "mod.ts",
-      r#"const jsonData = await Promise.resolve({ default: JSON.parse(`{ "prop": 5 }`) });"#
-    ),]
+    &[
+      (
+        "mod.ts",
+        r#"const jsonData = (await import('./data.js')).default;"#
+      ),
+      ("data.js", r#"export default JSON.parse(`{ "prop": 5 }`);"#)
+    ]
   );
 }
 
 #[tokio::test]
-async fn json_module_re_export_not_supported() {
-  let err = TestBuilder::new()
+async fn json_module_re_export() {
+  let result = TestBuilder::new()
     .with_loader(|loader| {
       loader
         .add_local_file(
@@ -1305,18 +1646,43 @@ async fn json_module_re_export_not_supported() {
     })
     .transform()
     .await
-    .err()
     .unwrap();
 
-  assert_eq!(
-    format!("{:?}", err),
-    concat!(
-      "Issue getting text changes from file:///mod.ts\n",
-      "\n",
-      "Caused by:\n",
-      "    Re-exporting JSON modules has not been implemented. If you need this functionality, please open an issue in dnt's repo.\n",
-      "    \n",
-      "    As a workaround, consider importing the module with an import declaration then exporting the identifier on a separate statement."
-    )
+  assert_files!(
+    result.main.files,
+    &[
+      ("mod.ts", r#"export { default as Test } from './data.js';"#),
+      ("data.js", r#"export default JSON.parse(`{ "prop": 5 }`);"#)
+    ]
   );
 }
+
+#[tokio::test]
+async fn issue_104() {
+  let result = TestBuilder::new()
+    .with_loader(|loader| {
+      loader
+        .add_local_file("/mod.ts", "import type { other } from './test.ts'; import { test } from './test.ts'; test();")
+        .add_local_file("/test.ts", "export function test() {} export type other = string;");
+    })
+    .transform()
+    .await
+    .unwrap();
+
+  assert_files!(
+    result.main.files,
+    &[
+      ("mod.ts", "import type { other } from './test.js'; import { test } from './test.js'; test();"),
+      ("test.ts", "export function test() {} export type other = string;"),
+    ]
+  );
+}
+
+fn get_shim_file_text(mut text: String) -> String {
+  text.push_str("\n");
+  text.push_str(
+    &include_str!("../src/scripts/createMergeProxy.ts")
+      .replace("export function", "function"),
+  );
+  text
+}
